@@ -8,7 +8,7 @@
 %%%
 %%% BSD LICENSE
 %%% 
-%%% Copyright (c) 2013-2015, Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2013-2017, Michael Truog <mjtruog at gmail dot com>
 %%% All rights reserved.
 %%% 
 %%% Redistribution and use in source and binary forms, with or without
@@ -43,8 +43,8 @@
 %%% DAMAGE.
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2015 Michael Truog
-%%% @version 1.5.1 {@date} {@time}
+%%% @copyright 2013-2017 Michael Truog
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi).
@@ -106,7 +106,9 @@
 -type response() :: any().
 -type timeout_value_milliseconds() :: 0..?TIMEOUT_MAX.
 -type timeout_milliseconds() :: timeout_value_milliseconds() |
-                                undefined | immediate.
+                                undefined | limit_min | limit_max |
+                                % deprecated type usage XXX
+                                immediate.
 -type priority_value() :: cloudi_service_api:priority().
 -type priority() :: priority_value() | undefined.
 -type trans_id() :: <<_:128>>. % version 1 UUID
@@ -163,8 +165,10 @@
             dest_refresh_delay
                 :: cloudi_service_api:dest_refresh_delay_milliseconds(),
             request_name_lookup :: sync | async,
-            timeout_async :: cloudi_service_api:timeout_milliseconds(),
-            timeout_sync :: cloudi_service_api:timeout_milliseconds(),
+            timeout_async
+                :: cloudi_service_api:timeout_send_async_value_milliseconds(),
+            timeout_sync
+                :: cloudi_service_api:timeout_send_sync_value_milliseconds(),
             priority_default :: cloudi_service_api:priority(),
             scope :: atom(),
             receiver :: pid(),
@@ -179,8 +183,10 @@
          {dest_refresh_delay,
           cloudi_service_api:dest_refresh_delay_milliseconds()} |
          {request_name_lookup, sync | async} |
-         {timeout_async, cloudi_service_api:timeout_milliseconds()} |
-         {timeout_sync, cloudi_service_api:timeout_milliseconds()} |
+         {timeout_async,
+          cloudi_service_api:timeout_send_async_value_milliseconds()} |
+         {timeout_sync,
+          cloudi_service_api:timeout_send_sync_value_milliseconds()} |
          {priority_default, priority()} |
          {scope, atom()} |
          % advanced:
@@ -416,8 +422,14 @@ get_pid(#cloudi_context{timeout_sync = DefaultTimeoutSync} = Context,
     get_pid(Context, Name, DefaultTimeoutSync);
 
 get_pid(#cloudi_context{} = Context, Name, immediate) ->
+    get_pid(Context, Name, limit_min);
+
+get_pid(#cloudi_context{} = Context, Name, limit_min) ->
     get_pid(Context, Name, ?SEND_SYNC_INTERVAL - 1);
 
+get_pid(#cloudi_context{} = Context, Name, limit_max) ->
+    get_pid(Context, Name, ?TIMEOUT_MAX);
+
 get_pid(#cloudi_context{} = Context, [NameC | _] = Name, Timeout)
     when is_integer(NameC), is_integer(Timeout),
          Timeout >= 0, Timeout =< ?TIMEOUT_MAX ->
@@ -484,8 +496,14 @@ get_pids(#cloudi_context{timeout_sync = DefaultTimeoutSync} = Context,
     get_pids(Context, Name, DefaultTimeoutSync);
 
 get_pids(#cloudi_context{} = Context, Name, immediate) ->
+    get_pids(Context, Name, limit_min);
+
+get_pids(#cloudi_context{} = Context, Name, limit_min) ->
     get_pids(Context, Name, ?SEND_SYNC_INTERVAL - 1);
 
+get_pids(#cloudi_context{} = Context, Name, limit_max) ->
+    get_pids(Context, Name, ?TIMEOUT_MAX);
+
 get_pids(#cloudi_context{} = Context, [NameC | _] = Name, Timeout)
     when is_integer(NameC), is_integer(Timeout),
          Timeout >= 0, Timeout =< ?TIMEOUT_MAX ->
@@ -634,9 +652,21 @@ send_async(#cloudi_context{timeout_async = DefaultTimeoutAsync} = Context,
 send_async(#cloudi_context{} = Context,
            Name, RequestInfo, Request,
            immediate, Priority, PatternPid) ->
+    send_async(Context, Name, RequestInfo, Request,
+               limit_min, Priority, PatternPid);
+
+send_async(#cloudi_context{} = Context,
+           Name, RequestInfo, Request,
+           limit_min, Priority, PatternPid) ->
     send_async(Context, Name, RequestInfo, Request,
                ?SEND_ASYNC_INTERVAL - 1, Priority, PatternPid);
 
+send_async(#cloudi_context{} = Context,
+           Name, RequestInfo, Request,
+           limit_max, Priority, PatternPid) ->
+    send_async(Context, Name, RequestInfo, Request,
+               ?TIMEOUT_MAX, Priority, PatternPid);
+
 send_async(#cloudi_context{priority_default = PriorityDefault} = Context,
            Name, RequestInfo, Request,
            Timeout, undefined, PatternPid) ->
@@ -914,9 +944,21 @@ send_sync(#cloudi_context{timeout_sync = DefaultTimeoutSync} = Context,
 send_sync(#cloudi_context{} = Context,
           Name, RequestInfo, Request,
           immediate, Priority, PatternPid) ->
+    send_sync(Context, Name, RequestInfo, Request,
+              limit_min, Priority, PatternPid);
+
+send_sync(#cloudi_context{} = Context,
+          Name, RequestInfo, Request,
+          limit_min, Priority, PatternPid) ->
     send_sync(Context, Name, RequestInfo, Request,
               ?SEND_SYNC_INTERVAL - 1, Priority, PatternPid);
 
+send_sync(#cloudi_context{} = Context,
+          Name, RequestInfo, Request,
+          limit_max, Priority, PatternPid) ->
+    send_sync(Context, Name, RequestInfo, Request,
+              ?TIMEOUT_MAX, Priority, PatternPid);
+
 send_sync(#cloudi_context{priority_default = PriorityDefault} = Context,
           Name, RequestInfo, Request,
           Timeout, undefined, PatternPid) ->
@@ -1045,9 +1087,19 @@ mcast_async(#cloudi_context{timeout_async = DefaultTimeoutAsync} = Context,
 
 mcast_async(#cloudi_context{} = Context,
             Name, RequestInfo, Request, immediate, Priority) ->
+    mcast_async(Context, Name, RequestInfo, Request,
+                limit_min, Priority);
+
+mcast_async(#cloudi_context{} = Context,
+            Name, RequestInfo, Request, limit_min, Priority) ->
     mcast_async(Context, Name, RequestInfo, Request,
                 ?MCAST_ASYNC_INTERVAL - 1, Priority);
 
+mcast_async(#cloudi_context{} = Context,
+            Name, RequestInfo, Request, limit_max, Priority) ->
+    mcast_async(Context, Name, RequestInfo, Request,
+                ?TIMEOUT_MAX, Priority);
+
 mcast_async(#cloudi_context{priority_default = PriorityDefault} = Context,
             Name, RequestInfo, Request, Timeout, undefined) ->
     mcast_async(Context, Name, RequestInfo, Request,
@@ -1188,8 +1240,14 @@ recv_async(Context, undefined) ->
     recv_async(Context, undefined, <<0:128>>);
 
 recv_async(Context, immediate) ->
+    recv_async(Context, limit_min);
+
+recv_async(Context, limit_min) ->
     recv_async(Context, ?RECV_ASYNC_INTERVAL - 1, <<0:128>>);
 
+recv_async(Context, limit_max) ->
+    recv_async(Context, ?TIMEOUT_MAX, <<0:128>>);
+
 recv_async(Context, Timeout)
     when is_integer(Timeout), Timeout >= 0 ->
     recv_async(Context, Timeout, <<0:128>>).
@@ -1219,8 +1277,14 @@ recv_async(#cloudi_context{timeout_sync = DefaultTimeoutSync} = Context,
     recv_async(Context, DefaultTimeoutSync, TransId);
 
 recv_async(#cloudi_context{} = Context, immediate, TransId) ->
+    recv_async(Context, limit_min, TransId);
+
+recv_async(#cloudi_context{} = Context, limit_min, TransId) ->
     recv_async(Context, ?RECV_ASYNC_INTERVAL - 1, TransId);
 
+recv_async(#cloudi_context{} = Context, limit_max, TransId) ->
+    recv_async(Context, ?TIMEOUT_MAX, TransId);
+
 recv_async(#cloudi_context{receiver = Receiver} = Context,
            Timeout, <<0:128>>)
     when is_integer(Timeout), Timeout >= 0 ->
@@ -1286,8 +1350,14 @@ recv_asyncs(#cloudi_context{timeout_sync = DefaultTimeoutSync} = Context,
     recv_asyncs(Context, DefaultTimeoutSync, TransIdList);
 
 recv_asyncs(#cloudi_context{} = Context, immediate, TransIdList) ->
+    recv_asyncs(Context, limit_min, TransIdList);
+
+recv_asyncs(#cloudi_context{} = Context, limit_min, TransIdList) ->
     recv_asyncs(Context, ?RECV_ASYNC_INTERVAL - 1, TransIdList);
 
+recv_asyncs(#cloudi_context{} = Context, limit_max, TransIdList) ->
+    recv_asyncs(Context, ?TIMEOUT_MAX, TransIdList);
+
 recv_asyncs(#cloudi_context{receiver = Receiver} = Context,
             Timeout, TransIdList)
     when is_integer(Timeout), is_list(TransIdList),
@@ -1309,7 +1379,7 @@ recv_asyncs(#cloudi_context{receiver = Receiver} = Context,
 %%-------------------------------------------------------------------------
 
 -spec timeout_async(Context :: agent()) ->
-    TimeoutAsync :: cloudi_service_api:timeout_milliseconds().
+    TimeoutAsync :: cloudi_service_api:timeout_send_async_value_milliseconds().
 
 timeout_async(Dispatcher)
     when is_pid(Dispatcher) ->
@@ -1325,7 +1395,7 @@ timeout_async(#cloudi_context{timeout_async = DefaultTimeoutAsync}) ->
 %%-------------------------------------------------------------------------
 
 -spec timeout_sync(Context :: agent()) ->
-    TimeoutSync :: cloudi_service_api:timeout_milliseconds().
+    TimeoutSync :: cloudi_service_api:timeout_send_sync_value_milliseconds().
 
 timeout_sync(Dispatcher)
     when is_pid(Dispatcher) ->
@@ -1341,7 +1411,7 @@ timeout_sync(#cloudi_context{timeout_sync = DefaultTimeoutSync}) ->
 %%-------------------------------------------------------------------------
 
 -spec timeout_max(Context :: agent()) ->
-    TimeoutMax :: cloudi_service_api:timeout_milliseconds().
+    TimeoutMax :: pos_integer().
 
 timeout_max(Dispatcher)
     when is_pid(Dispatcher) ->
