@@ -45,7 +45,7 @@
 %%%
 %%% @author Michael Truog <mjtruog [at] gmail (dot) com>
 %%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% @version 1.5.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -70,6 +70,7 @@
          nodes_get/1,
          nodes_add/2,
          nodes_remove/2,
+         logging_set/2,
          logging_file_set/2,
          logging_level_set/2,
          logging_syslog_set/2,
@@ -131,6 +132,11 @@
 -define(CATCH_EXIT(F),
         try F catch exit:{Reason, _} -> {error, Reason} end).
 
+-ifdef(CLOUDI_CORE_STANDALONE).
+-dialyzer({no_match,
+           [service_update_external/6]}).
+-endif.
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -210,6 +216,11 @@ nodes_remove(L, Timeout) ->
                                 {nodes_remove, L, local,
                                  timeout_decr(Timeout)}, Timeout)).
 
+logging_set(L, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {logging_set, L,
+                                 timeout_decr(Timeout)}, Timeout)).
+
 logging_file_set(FilePath, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {logging_file_set, FilePath,
@@ -462,6 +473,20 @@ handle_call({nodes_add, _, _, _} = Request, _, State) ->
 handle_call({nodes_remove, _, _, _} = Request, _, State) ->
     nodes_call(Request, State);
 
+handle_call({logging_set, L, _}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:logging_set(L, Config) of
+        {ok, #config{logging = LoggingConfig} = NewConfig} ->
+            case cloudi_core_i_logger:set(LoggingConfig) of
+                ok ->
+                    {reply, ok, State#state{configuration = NewConfig}};
+                {error, _} = Error ->
+                    {reply, Error, State}
+            end;
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
 handle_call({logging_file_set, FilePath, _}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
@@ -495,13 +520,12 @@ handle_call({logging_syslog_set, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({logging_formatters_set, L, Timeout}, _,
+handle_call({logging_formatters_set, L, _}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_formatters_set(L, Config) of
         {ok, #config{logging = #config_logging{
                          formatters = FormattersConfig}} = NewConfig} ->
-            ok = cloudi_core_i_logger:formatters_set(FormattersConfig,
-                                                     Timeout),
+            ok = cloudi_core_i_logger:formatters_set(FormattersConfig),
             {reply, ok, State#state{configuration = NewConfig}};
         {error, _} = Error ->
             {reply, Error, State}
@@ -510,7 +534,7 @@ handle_call({logging_formatters_set, L, Timeout}, _,
 handle_call({logging_redirect_set, Node, _}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
-    ok = cloudi_core_i_nodes:logging_redirect_set(Node),
+    ok = cloudi_core_i_logger:redirect_set(Node),
     NewConfig = Config#config{
                     logging = LoggingConfig#config_logging{
                         redirect = Node}},
@@ -592,11 +616,9 @@ configure_service([Service | Services], Configured, Timeout) ->
         {ok, NewService} ->
             configure_service(Services, [NewService | Configured], Timeout);
         {error, Reason} = Error ->
-            ServiceDescription = cloudi_core_i_configuration:
-                                 service_format(Service),
             % wait for logging statements to be logged before crashing
             ?LOG_FATAL_SYNC("configure failed: ~p~n~p",
-                            [Reason, ServiceDescription]),
+                            [Reason, service_format(Service)]),
             Error
     end.
 
@@ -961,6 +983,7 @@ service_start_internal(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
@@ -974,12 +997,9 @@ service_start_internal(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, 1, Scope,
-                 TimeoutTerm, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
-            {ID, ServiceConfig} = cloudi_core_i_configuration:
-                                  service_format(Service),
-            ?LOG_INFO("~p -> ~p", [{service_id(ID),
-                                    ServiceConfig}, P]),
+            ?LOG_INFO("~p -> ~p", [service_format(Service), P]),
             service_start_internal(IndexProcess + 1, [P | Pids], Service,
                                    GroupLeader, CountProcess, Timeout);
         {error, Reason} ->
@@ -1008,6 +1028,7 @@ service_start_external(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
                            max_t = MaxT,
@@ -1022,12 +1043,9 @@ service_start_external(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, CountThread, Scope,
-                 TimeoutTerm, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
-            {ID, ServiceConfig} = cloudi_core_i_configuration:
-                                  service_format(Service),
-            ?LOG_INFO("~p -> ~p", [{service_id(ID),
-                                    ServiceConfig}, P]),
+            ?LOG_INFO("~p -> ~p", [service_format(Service), P]),
             service_start_external(IndexProcess + 1, [P | Pids], Service,
                                    CountThread, CountProcess, Timeout);
         {error, Reason} ->
@@ -1196,6 +1214,10 @@ timeout_decr(Timeout)
     when is_integer(Timeout), Timeout >= ?TIMEOUT_DELTA ->
     Timeout - ?TIMEOUT_DELTA.
 
+service_format(Service) ->
+    {ID, ServiceConfig} = cloudi_core_i_configuration:service_format(Service),
+    {service_id(ID), ServiceConfig}.
+
 service_id(ID) ->
     cloudi_x_uuid:uuid_to_string(ID, list_nodash).
 
